@@ -108,7 +108,9 @@ namespace PoDoFo
 				char *filenameBuffer = new char[1000];
 				do
 				{
-					in.getline ( filenameBuffer, 1000 );
+					if ( !in.getline ( filenameBuffer, 1000 ) )
+						throw runtime_error ( "failed reading line from input file" );
+
 					std::string ts ( filenameBuffer, in.gcount() );
 					if ( ts.size() > 4 ) // at least ".pdf" because just test if ts is empty doesn't work.
 					{
@@ -122,6 +124,9 @@ namespace PoDoFo
 			}
 			std::cerr<< ++dbg <<std::endl;
 
+			if (multiSource.empty())
+				throw runtime_error( "No recognized source given" );
+
 			for ( std::vector<std::string>::const_iterator ms = multiSource.begin(); ms != multiSource.end(); ++ms )
 			{
 				if ( ms == multiSource.begin() )
@@ -148,7 +153,12 @@ namespace PoDoFo
 // 	std::cerr << "Document has "<< pcount << " page(s) " << endl;
 			if ( pcount > 0 ) // only here to avoid possible segfault, but PDF without page is not conform IIRC
 			{
-				PoDoFo::PdfRect rect ( sourceDoc->GetPage ( 0 )->GetMediaBox() );
+                PoDoFo::PdfPage* pFirstPage = sourceDoc->GetPage ( 0 );
+                if ( NULL == pFirstPage ) // Fixes CVE-2019-9199 (issue #40)
+                {
+                    PODOFO_RAISE_ERROR_INFO( ePdfError_PageNotFound, "First page (0) of source document not found" );
+                }
+                PoDoFo::PdfRect rect ( pFirstPage->GetMediaBox() );
 				// keep in mind it’s just a hint since PDF can have different page sizes in a same doc
 				sourceWidth =  rect.GetWidth() - rect.GetLeft();
 				sourceHeight =  rect.GetHeight() - rect.GetBottom() ;
@@ -167,20 +177,47 @@ namespace PoDoFo
 
 		}
 
-		PdfObject* PdfTranslator::migrateResource ( PdfObject * obj )
+		PdfObject* PdfTranslator::migrateResource ( const PdfObject * obj )
 		{
 // 			std::cerr<<"PdfTranslator::migrateResource"<<std::endl;
 			PdfObject *ret ( 0 );
 
+			if ( !obj )
+				PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidHandle, "migrateResource called"
+                                         " with NULL object" );
+
 			if ( obj->IsDictionary() )
 			{
-				ret = targetDoc->GetObjects().CreateObject ( *obj );
-
+				if ( obj->Reference().IsIndirect() )
+				{
+					ret = targetDoc->GetObjects().CreateObject ( *obj );
+				}
+				else
+				{
+					ret = new PdfObject( *obj );
+				}
 				TKeyMap resmap = obj->GetDictionary().GetKeys();
 				for ( TCIKeyMap itres = resmap.begin(); itres != resmap.end(); ++itres )
 				{
 					PdfObject *o = itres->second;
-					ret->GetDictionary().AddKey ( itres->first , migrateResource ( o ) );
+					std::pair<std::set<PdfObject*>::iterator,bool> res = setMigrationPending.insert( o );
+					if (!res.second)
+					{
+						std::ostringstream oss;
+						oss << "Cycle detected: Object with ref " << o->Reference().ToString()
+							<< " is already pending migration to the target.\n";
+						PdfError::LogMessage( eLogSeverity_Warning, oss.str().c_str() );
+						continue;
+					}
+					PdfObject *migrated = migrateResource ( o );
+					if (NULL != migrated)
+					{
+						ret->GetDictionary().AddKey ( itres->first, migrated );
+						if ( !(migrated->Reference().IsIndirect()) )
+						{
+							delete migrated;
+						}
+					}
 				}
 
 				if ( obj->HasStream() )
@@ -195,51 +232,81 @@ namespace PoDoFo
 				for ( unsigned int ci = 0; ci < carray.GetSize(); ++ci )
 				{
 					PdfObject *co ( migrateResource ( &carray[ci] ) );
+					if ( NULL == co )
+						continue;
 					narray.push_back ( *co );
+
+					if ( !(co->Reference().IsIndirect()) )
+					{
+						delete co;
+					}
+				}
+				if ( obj->Reference().IsIndirect() )
+				{
+					ret = targetDoc->GetObjects().CreateObject ( narray );
+				}
+				else
+				{
+					ret = new PdfObject( narray );
 				}
-				ret = targetDoc->GetObjects().CreateObject ( narray );
 			}
 			else if ( obj->IsReference() )
 			{
 				if ( migrateMap.find ( obj->GetReference().ToString() ) != migrateMap.end() )
 				{
-					return migrateMap[obj->GetReference().ToString() ];
+					std::ostringstream oss;
+					oss << "Referenced object " << obj->GetReference().ToString()
+					    << " already migrated." << std::endl;
+					PdfError::DebugMessage( oss.str().c_str() );
+
+					const PdfObject* const found = migrateMap[ obj->GetReference().ToString() ];
+					return new PdfObject( found->Reference() );
 				}
 
-				PdfObject * o ( migrateResource ( sourceDoc->GetObjects().GetObject ( obj->GetReference() ) ) );
-
-                                ret  = new PdfObject ( o->Reference() ) ;
+				PdfObject *to_migrate = sourceDoc->GetObjects().GetObject ( obj->GetReference() );
 
+				std::pair<std::set<PdfObject*>::iterator, bool> res
+						= setMigrationPending.insert( to_migrate );
+				if (!res.second)
+				{
+					std::ostringstream oss;
+					oss << "Cycle detected: Object with ref " << obj->GetReference().ToString()
+						<< " is already pending migration to the target.\n";
+					PdfError::LogMessage( eLogSeverity_Warning, oss.str().c_str() );	
+					return NULL; // skip this migration
+				}
+				PdfObject * o ( migrateResource ( to_migrate ) );
+				if ( NULL != o )
+					ret  = new PdfObject ( o->Reference() );
+				else
+					return NULL; // avoid going through rest of method
 			}
 			else
 			{
 				ret = new PdfObject ( *obj );//targetDoc->GetObjects().CreateObject(*obj);
 			}
 
-
-			migrateMap.insert ( std::pair<std::string, PdfObject*> ( obj->Reference().ToString(), ret ) );
-
+			if ( obj->Reference().IsIndirect() )
+			{
+				migrateMap.insert ( std::pair<std::string, PdfObject*> ( obj->Reference().ToString(), ret ) );
+			}
 
 			return ret;
-
 		}
 
 		PdfObject* PdfTranslator::getInheritedResources ( PdfPage* page )
 		{
 // 			std::cerr<<"PdfTranslator::getInheritedResources"<<std::endl;
-			PdfObject *res ( 0 ); // = new PdfObject;
-			PdfObject *rparent = page->GetObject();
-			while ( rparent && rparent->IsDictionary() )
+			PdfObject *res ( 0 );
+			// mabri: resources are inherited as whole dict, not at all if the page has the dict
+			// mabri: specified in PDF32000_2008.pdf section 7.7.3.4 Inheritance of Page Attributes
+			// mabri: and in section 7.8.3 Resource Dictionaries
+			const PdfObject *sourceRes = page->GetInheritedKey( PdfName ( "Resources" ) );
+			if ( sourceRes )
 			{
-				PdfObject *curRes = rparent->GetDictionary().GetKey ( PdfName ( "Resources" ) );
-				if ( curRes )
-				{
-					res = migrateResource ( curRes );
-				}
-				rparent = rparent->GetIndirectKey ( "Parent" );
+			    res = migrateResource( sourceRes );
 			}
 			return res;
-
 		}
 
 		void PdfTranslator::setTarget ( const std::string & target )
@@ -256,7 +323,13 @@ namespace PoDoFo
 				PdfPage * page = sourceDoc->GetPage ( i );
 				PdfMemoryOutputStream outMemStream ( 1 );
 
-				PdfXObject *xobj = new PdfXObject ( page->GetMediaBox(), targetDoc );
+				if (!page) // Fix issue #32
+                {
+                    std::ostringstream oss;
+                    oss << "Page " << i << " (0-based) of " << pcount << " in source doc not found!";
+                    PODOFO_RAISE_ERROR_INFO( ePdfError_PageNotFound, oss.str() );
+                }
+                PdfXObject *xobj = new PdfXObject ( page->GetMediaBox(), targetDoc );
 				if ( page->GetContents()->HasStream() )
 				{
 					page->GetContents()->GetStream()->GetFilteredCopy ( &outMemStream );
@@ -301,7 +374,10 @@ namespace PoDoFo
 					PoDoFo::PdfName keyname ( *itKey );
 					if ( page->GetObject()->GetDictionary().HasKey ( keyname ) )
 					{
-						xobj->GetObject()->GetDictionary().AddKey ( keyname, migrateResource ( page->GetObject()->GetDictionary().GetKey ( keyname ) ) );
+						PdfObject* migObj = migrateResource ( page->GetObject()->GetDictionary().GetKey ( keyname ) );
+						if ( NULL == migObj )
+							continue;
+						xobj->GetObject()->GetDictionary().AddKey ( keyname, migObj ); 
 					}
 				}
 
@@ -459,6 +535,7 @@ namespace PoDoFo
 						{
 							PdfObject bb;
 							// DominikS: Fix compilation using Visual Studio on Windows
+							// mabri: ML post archive URL is https://sourceforge.net/p/podofo/mailman/message/24609746/
 							// bbIndex->at(resourceIndex).ToVariant( bb );							
 							((*bbIndex)[resourceIndex]).ToVariant( bb );
 							xo->GetObject()->GetDictionary().AddKey ( PdfName ( "BBox" ), bb );
@@ -503,6 +580,13 @@ namespace PoDoFo
 
 			targetDoc->Write ( outFilePath.c_str() );
 
+			// The following is necessary to avoid line 195 being detected as allocation having a memory leak
+			// without changing other files than this one (thorough leak prevention shall be applied later).
+			for (std::map<int, PdfObject*>::iterator it = resources.begin(); it != resources.end(); it++)
+			{
+				delete (*it).second;
+			}
+			resources.clear(); 
 		}
 
 
